from pydicom.sr.codedict import codes
from pydicom.sr.coding import Code


class TestCodeDict:
    def test_dcm_1(self):
        assert codes.DCM.Modality == Code(
            value="121139", scheme_designator="DCM", meaning="Modality"
        )

    def test_dcm_2(self):
        assert codes.DCM.ProcedureReported == Code(
            value="121058",
            scheme_designator="DCM",
            meaning="Procedure Reported",
        )

    def test_dcm_3(self):
        assert codes.DCM.ImagingStartDatetime == Code(
            value="122712",
            scheme_designator="DCM",
            meaning="Imaging Start DateTime",
        )

    def test_sct_1(self):
        assert codes.SCT._1SigmaLowerValueOfPopulation == Code(
            value="371919006",
            scheme_designator="SCT",
            meaning="1 Sigma Lower Value of Populuation",
        )

    def test_sct_2(self):
        assert codes.SCT.FindingSite == Code(
            value="363698007", scheme_designator="SCT", meaning="Finding Site"
        )

    def test_cid250(self):
        assert codes.cid250.Positive == Code(
            value="10828004", scheme_designator="SCT", meaning="Positive"
        )

    def test_cid300(self):
        assert codes.cid300.NickelCobaltChromium == Code(
            value="261249004",
            scheme_designator="SCT",
            meaning="Nickel cobalt chromium",
        )

    def test_cid301(self):
        assert codes.cid301.mgcm3 == Code(
            value="mg/cm3", scheme_designator="UCUM", meaning="mg/cm^3"
        )

    def test_cid402(self):
        assert codes.cid402.DestinationRoleID == Code(
            value="110152",
            scheme_designator="DCM",
            meaning="Destination Role ID",
        )

    def test_cid405(self):
        assert codes.cid405.MultiMediaCard == Code(
            value="110035", scheme_designator="DCM", meaning="Multi-media Card"
        )

    def test_cid610(self):
        assert codes.cid610.ReverseOsmosisPurifiedHclAcidifiedWater == Code(
            value="127291",
            scheme_designator="DCM",
            meaning="Reverse osmosis purified, HCl acidified water",
        )

    def test_cid612(self):
        assert codes.cid612.MonitoredAnesthesiaCareMAC == Code(
            value="398239001",
            scheme_designator="SCT",
            meaning="Monitored Anesthesia Care (MAC)",
        )

    def test_cid622(self):
        assert codes.cid622.NeuromuscularBlockingNMBNonDepolarizing == Code(
            value="372790002",
            scheme_designator="SCT",
            meaning="NeuroMuscular Blocking (NMB) - non depolarizing",
        )

    def test_cid630(self):
        assert codes.cid630.LidocainePrilocaine == Code(
            value="346553009",
            scheme_designator="SCT",
            meaning="Lidocaine + Prilocaine",
        )

    def test_cid643(self):
        assert codes.cid643._6Hydroxydopamine == Code(
            value="4624",
            scheme_designator="PUBCHEM_CID",
            meaning="6-Hydroxydopamine",
        )

    def test_cid646(self):
        assert codes.cid646.SPECTCTOfWholeBody == Code(
            value="127902",
            scheme_designator="DCM",
            meaning="SPECT CT of Whole Body",
        )

    def test_cid1003(self):
        assert codes.cid1003.LevelOfT11T12IntervertebralDisc == Code(
            value="243918001",
            scheme_designator="SCT",
            meaning="Level of T11/T12 intervertebral disc",
        )

    def test_cid3000(self):
        assert codes.cid3000.OperatorNarrative == Code(
            value="109111",
            scheme_designator="DCM",
            meaning="Operator's Narrative",
        )

    def test_cid3001_1(self):
        assert codes.cid3001.Avr == Code(
            value="2:65", scheme_designator="MDC", meaning="-aVR"
        )

    def test_cid3001_2(self):
        assert codes.cid3001.NegativeLowRightScapulaLead == Code(
            value="2:124",
            scheme_designator="MDC",
            meaning="negativ: low right scapula Lead",
        )

    def test_cid3107(self):
        assert codes.cid3107._13Nitrogen == Code(
            value="21576001", scheme_designator="SCT", meaning="^13^Nitrogen"
        )

    def test_cid3111(self):
        assert codes.cid3111.Tc99mTetrofosmin == Code(
            value="404707004",
            scheme_designator="SCT",
            meaning="Tc-99m tetrofosmin",
        )

    def test_cid3263(self):
        meaning = (
            "12-lead from EASI leads (ES, AS, AI)"
            " by Dower/EASI transformation"
        )
        assert (
            codes.cid3263._12LeadFromEASILeadsESASAIByDowerEASITransformation
            == Code(
                value="10:11284", scheme_designator="MDC", meaning=meaning,
            )
        )

    def test_cid3335(self):
        assert codes.cid3335.PWaveSecondDeflectionInPWave == Code(
            value="10:320",
            scheme_designator="MDC",
            meaning="P' wave (second deflection in P wave)",
        )

    def test_contained(self):
        c = Code("24028007", "SCT", "Right")
        assert c in codes.cid244

    def test_not_contained(self):
        c = Code("130290", "DCM", "Median")
        assert c not in codes.cid244



# Copyright 2020 pydicom authors. See LICENSE file for details.
"""Tests for the pixel_data_handlers.pylibjpeg_handler module."""

import pytest

import pydicom
from pydicom.data import get_testdata_file
from pydicom.encaps import defragment_data
from pydicom.filereader import dcmread
from pydicom.pixel_data_handlers.util import (
    convert_color_space, get_j2k_parameters
)
from pydicom.uid import (
    ImplicitVRLittleEndian,
    JPEGBaseline8Bit,
    JPEGExtended12Bit,
    JPEGLosslessP14,
    JPEGLosslessSV1,
    JPEGLSLossless,
    JPEGLSNearLossless,
    JPEG2000Lossless,
    JPEG2000,
    AllTransferSyntaxes
)

try:
    import numpy as np
    from pydicom.pixel_data_handlers import numpy_handler as NP_HANDLER
    HAVE_NP = True
except ImportError:
    NP_HANDLER = None
    HAVE_NP = False

try:
    from pydicom.pixel_data_handlers import pylibjpeg_handler as LJ_HANDLER
    from pydicom.pixel_data_handlers.pylibjpeg_handler import (
        get_pixeldata, as_array, generate_frames
    )
    HAVE_PYLIBJPEG = LJ_HANDLER.HAVE_PYLIBJPEG
    HAVE_LJ = LJ_HANDLER.HAVE_LIBJPEG
    HAVE_OJ = LJ_HANDLER.HAVE_OPENJPEG
except ImportError:
    LJ_HANDLER = None
    HAVE_PYLIBJPEG = False
    HAVE_LJ = False
    HAVE_OJ = False


TEST_HANDLER = HAVE_NP and HAVE_PYLIBJPEG  # Run handler tests
TEST_JPEG = TEST_HANDLER and HAVE_LJ  # Run 10918 JPEG tests
TEST_JPEGLS = TEST_HANDLER and HAVE_LJ  # Run 14495 JPEG-LS tests
TEST_JPEG2K = TEST_HANDLER and HAVE_OJ  # Run 15444 JPEG 2000 tests


SUPPORTED_SYNTAXES = [
    JPEGBaseline8Bit,
    JPEGExtended12Bit,
    JPEGLosslessP14,
    JPEGLosslessSV1,
    JPEGLSLossless,
    JPEGLSNearLossless,
    JPEG2000Lossless,
    JPEG2000
]
UNSUPPORTED_SYNTAXES = list(
    set(AllTransferSyntaxes) ^ set(SUPPORTED_SYNTAXES)
)

# Transfer syntaxes supported by other handlers
IMPL = get_testdata_file("MR_small_implicit.dcm")
EXPL = get_testdata_file("OBXXXX1A.dcm")
EXPB = get_testdata_file("OBXXXX1A_expb.dcm")
DEFL = get_testdata_file("image_dfl.dcm")
RLE = get_testdata_file("MR_small_RLE.dcm")

REFERENCE_DATA_UNSUPPORTED = [
    (IMPL, ('1.2.840.10008.1.2', 'CompressedSamples^MR1')),
    (EXPL, ('1.2.840.10008.1.2.1', 'OB^^^^')),
    (EXPB, ('1.2.840.10008.1.2.2', 'OB^^^^')),
    (DEFL, ('1.2.840.10008.1.2.1.99', '^^^^')),
    (RLE, ('1.2.840.10008.1.2.5', 'CompressedSamples^MR1')),
]

# JPEG - ISO/IEC 10918 Standard
# FMT_BA_BV_SPX_PR_FRAMESF_PI
# JPGB: 1.2.840.10008.1.2.4.50 - JPEG Baseline (8-bit only)
JPGB_08_08_3_0_1F_YBR_FULL = get_testdata_file("SC_rgb_small_odd_jpeg.dcm")
JPGB_08_08_3_0_120F_YBR_FULL_422 = get_testdata_file("color3d_jpeg_baseline.dcm")  # noqa
# Different subsampling 411, 422, 444
JPGB_08_08_3_0_1F_YBR_FULL_422_411 = get_testdata_file("SC_rgb_dcmtk_+eb+cy+np.dcm")  # noqa
JPGB_08_08_3_0_1F_YBR_FULL_422_422 = get_testdata_file("SC_rgb_dcmtk_+eb+cy+s2.dcm")  # noqa
JPGB_08_08_3_0_1F_YBR_FULL_411 = get_testdata_file("SC_rgb_dcmtk_+eb+cy+n1.dcm")  # noqa
JPGB_08_08_3_0_1F_YBR_FULL_422 = get_testdata_file("SC_rgb_dcmtk_+eb+cy+n2.dcm")  # noqa
JPGB_08_08_3_0_1F_YBR_FULL_444 = get_testdata_file("SC_rgb_dcmtk_+eb+cy+s4.dcm")  # noqa
JPGB_08_08_3_0_1F_RGB = get_testdata_file("SC_rgb_dcmtk_+eb+cr.dcm")
# JPGE: 1.2.840.1.2.4.51 - JPEG Extended
JPGE_BAD = get_testdata_file("JPEG-lossy.dcm")  # Bad JPEG file
JPGE_16_12_1_0_1F_M2 = get_testdata_file("JPGExtended.dcm")  # Fixed version
# JPGL: 1.2.840.10008.1.2.4.70 - JPEG Lossless, Non-hierarchical, 1st Order
JPGL_08_08_1_0_1F = get_testdata_file("JPGLosslessP14SV1_1s_1f_8b.dcm")
JPGL_16_16_1_1_1F_M2 = get_testdata_file("JPEG-LL.dcm")

JPGB = JPEGBaseline8Bit
JPGE = JPEGExtended12Bit
JPGL = JPEGLosslessSV1

JPG_REFERENCE_DATA = [
    # fpath, (syntax, bits, nr samples, pixel repr, nr frames, shape, dtype)
    (JPGB_08_08_3_0_120F_YBR_FULL_422, (JPGB, 8, 3, 0, 120, (120, 480, 640, 3), 'uint8')),  # noqa
    (JPGB_08_08_3_0_1F_YBR_FULL_422_411, (JPGB, 8, 3, 0, 1, (100, 100, 3), 'uint8')),  # noqa
    (JPGB_08_08_3_0_1F_YBR_FULL_422_422, (JPGB, 8, 3, 0, 1, (100, 100, 3), 'uint8')),  # noqa
    (JPGB_08_08_3_0_1F_YBR_FULL_411, (JPGB, 8, 3, 0, 1, (100, 100, 3), 'uint8')),  # noqa
    (JPGB_08_08_3_0_1F_YBR_FULL_422, (JPGB, 8, 3, 0, 1, (100, 100, 3), 'uint8')),  # noqa
    (JPGB_08_08_3_0_1F_YBR_FULL_444, (JPGB, 8, 3, 0, 1, (100, 100, 3), 'uint8')),  # noqa
    (JPGB_08_08_3_0_1F_RGB, (JPGB, 8, 3, 0, 1, (100, 100, 3), 'uint8')),
    (JPGE_16_12_1_0_1F_M2, (JPGE, 16, 1, 0, 1, (1024, 256), 'uint16')),
    (JPGL_08_08_1_0_1F, (JPGL, 8, 1, 0, 1, (768, 1024), 'uint8')),
    (JPGL_16_16_1_1_1F_M2, (JPGL, 16, 1, 1, 1, (1024, 256), 'int16')),
]
JPG_MATCHING_DATASETS = [
    # (compressed, reference, hard coded check values), px tolerance
    pytest.param(
        JPGB_08_08_3_0_1F_YBR_FULL_422_411,
        get_testdata_file("SC_rgb_dcmtk_ebcynp_dcmd.dcm"),
        [
            (253, 1, 0), (253, 129, 131), (0, 255, 5), (127, 255, 129),
            (0, 0, 254), (127, 128, 255), (0, 0, 0), (64, 64, 64),
            (192, 192, 192), (255, 255, 255),
        ],
        2
    ),
    pytest.param(
        JPGB_08_08_3_0_1F_YBR_FULL_422_422,
        get_testdata_file("SC_rgb_dcmtk_ebcys2_dcmd.dcm"),
        [
            (254, 0, 0), (255, 127, 127), (0, 255, 5), (129, 255, 129),
            (0, 0, 254), (128, 127, 255), (0, 0, 0), (64, 64, 64),
            (192, 192, 192), (255, 255, 255),
        ],
        0
    ),
    pytest.param(
        JPGB_08_08_3_0_1F_YBR_FULL_411,
        get_testdata_file("SC_rgb_dcmtk_ebcyn1_dcmd.dcm"),
        [
            (253, 1, 0), (253, 129, 131), (0, 255, 5), (127, 255, 129),
            (0, 0, 254), (127, 128, 255), (0, 0, 0), (64, 64, 64),
            (192, 192, 192), (255, 255, 255),
        ],
        2
    ),
    pytest.param(
        JPGB_08_08_3_0_1F_YBR_FULL_422,
        get_testdata_file("SC_rgb_dcmtk_ebcyn2_dcmd.dcm"),
        [
            (254, 0, 0), (255, 127, 127), (0, 255, 5), (129, 255, 129),
            (0, 0, 254), (128, 127, 255), (0, 0, 0), (64, 64, 64),
            (192, 192, 192), (255, 255, 255),
        ],
        0
    ),
    pytest.param(
        JPGB_08_08_3_0_1F_YBR_FULL_444,
        get_testdata_file("SC_rgb_dcmtk_ebcys4_dcmd.dcm"),
        [
            (254, 0, 0), (255, 127, 127), (0, 255, 5), (129, 255, 129),
            (0, 0, 254), (128, 127, 255), (0, 0, 0), (64, 64, 64),
            (192, 192, 192), (255, 255, 255),
        ],
        0
    ),
    pytest.param(
        JPGB_08_08_3_0_1F_RGB,
        get_testdata_file("SC_rgb_dcmtk_ebcr_dcmd.dcm"),
        [
            (255, 0, 0), (255, 128, 128), (0, 255, 0), (128, 255, 128),
            (0, 0, 255), (128, 128, 255), (0, 0, 0), (64, 64, 64),
            (192, 192, 192), (255, 255, 255),
        ],
        1
    ),
]


# JPEG-LS - ISO/IEC 14495 Standard
JLSL = JPEGLSNearLossless
JLSN = JPEGLSLossless
JPEG_LS_LOSSLESS = get_testdata_file("MR_small_jpeg_ls_lossless.dcm")
JLS_REFERENCE_DATA = [
    # fpath, (syntax, bits, nr samples, pixel repr, nr frames, shape, dtype)
    (JPEG_LS_LOSSLESS, (JLSN, 16, 1, 1, 1, (64, 64), 'int16')),
]

# JPEG 2000 - ISO/IEC 15444 Standard
J2KR = JPEG2000Lossless
J2KI = JPEG2000
# J2KR: 1.2.840.100008.1.2.4.90 - JPEG 2000 Lossless
J2KR_08_08_3_0_1F_YBR_ICT = get_testdata_file("US1_J2KR.dcm")
J2KR_16_10_1_0_1F_M1 = get_testdata_file("RG3_J2KR.dcm")
J2KR_16_12_1_0_1F_M2 = get_testdata_file("MR2_J2KR.dcm")
J2KR_16_15_1_0_1F_M1 = get_testdata_file("RG1_J2KR.dcm")
J2KR_16_16_1_0_10F_M2 = get_testdata_file("emri_small_jpeg_2k_lossless.dcm")
J2KR_16_14_1_1_1F_M2 = get_testdata_file("693_J2KR.dcm")
J2KR_16_16_1_1_1F_M2 = get_testdata_file("MR_small_jp2klossless.dcm")
J2KR_16_13_1_1_1F_M2_MISMATCH = get_testdata_file("J2K_pixelrep_mismatch.dcm")
# Non-conformant pixel data -> JP2 header present
J2KR_08_08_3_0_1F_YBR_RCT = get_testdata_file("GDCMJ2K_TextGBR.dcm")
# J2KI: 1.2.840.10008.1.2.4.91 - JPEG 2000
J2KI_08_08_3_0_1F_RGB = get_testdata_file("SC_rgb_gdcm_KY.dcm")
J2KI_08_08_3_0_1F_YBR_ICT = get_testdata_file("US1_J2KI.dcm")
J2KI_16_10_1_0_1F_M1 = get_testdata_file("RG3_J2KI.dcm")
J2KI_16_12_1_0_1F_M2 = get_testdata_file("MR2_J2KI.dcm")
J2KI_16_15_1_0_1F_M1 = get_testdata_file("RG1_J2KI.dcm")
J2KI_16_14_1_1_1F_M2 = get_testdata_file("693_J2KI.dcm")
J2KI_16_16_1_1_1F_M2 = get_testdata_file("JPEG2000.dcm")

J2K_REFERENCE_DATA = [
    # fpath, (syntax, bits, nr samples, pixel repr, nr frames, shape, dtype)
    (J2KR_08_08_3_0_1F_YBR_ICT, (J2KR, 8, 3, 0, 1, (480, 640, 3), 'uint8')),
    (J2KR_16_10_1_0_1F_M1, (J2KR, 16, 1, 0, 1, (1760, 1760), 'uint16')),
    (J2KR_16_12_1_0_1F_M2, (J2KR, 16, 1, 0, 1, (1024, 1024), 'uint16')),
    (J2KR_16_15_1_0_1F_M1, (J2KR, 16, 1, 0, 1, (1955, 1841), 'uint16')),
    # should be Bits Stored = 12
    (J2KR_16_16_1_0_10F_M2, (J2KR, 16, 1, 0, 10, (10, 64, 64), 'uint16')),
    # should be Bits Stored = 16
    (J2KR_16_14_1_1_1F_M2, (J2KR, 16, 1, 1, 1, (512, 512), 'int16')),
    (J2KR_16_16_1_1_1F_M2, (J2KR, 16, 1, 1, 1, (64, 64), 'int16')),
    (J2KI_08_08_3_0_1F_RGB, (J2KI, 8, 3, 0, 1, (100, 100, 3), 'uint8')),
    (J2KI_08_08_3_0_1F_YBR_ICT, (J2KI, 8, 3, 0, 1, (480, 640, 3), 'uint8')),
    (J2KI_16_10_1_0_1F_M1, (J2KI, 16, 1, 0, 1, (1760, 1760), 'uint16')),
    (J2KI_16_12_1_0_1F_M2, (J2KI, 16, 1, 0, 1, (1024, 1024), 'uint16')),
    (J2KI_16_15_1_0_1F_M1, (J2KI, 16, 1, 0, 1, (1955, 1841), 'uint16')),
    # should be Bits Stored = 16
    (J2KI_16_14_1_1_1F_M2, (J2KI, 16, 1, 1, 1, (512, 512), 'int16')),
    (J2KI_16_16_1_1_1F_M2, (J2KI, 16, 1, 1, 1, (1024, 256), 'int16')),
]
J2K_MATCHING_DATASETS = [
    # (compressed, reference, fixes)
    pytest.param(
        J2KR_08_08_3_0_1F_YBR_ICT,
        get_testdata_file("US1_UNCR.dcm"),
        {},
    ),
    pytest.param(
        J2KR_16_10_1_0_1F_M1,
        get_testdata_file("RG3_UNCR.dcm"),
        {},
    ),
    pytest.param(
        J2KR_16_12_1_0_1F_M2,
        get_testdata_file("MR2_UNCR.dcm"),
        {},
    ),
    pytest.param(
        J2KR_16_15_1_0_1F_M1,
        get_testdata_file("RG1_UNCR.dcm"),
        {},
    ),
    pytest.param(
        J2KR_16_16_1_0_10F_M2,
        get_testdata_file("emri_small.dcm"),
        {'BitsStored': 16},
    ),
    pytest.param(
        J2KR_16_14_1_1_1F_M2,
        get_testdata_file("693_UNCR.dcm"),
        {'BitsStored': 14},
    ),
    pytest.param(
        J2KR_16_16_1_1_1F_M2,
        get_testdata_file("MR_small.dcm"),
        {},
    ),
    pytest.param(
        J2KI_08_08_3_0_1F_RGB,
        get_testdata_file("SC_rgb_gdcm2k_uncompressed.dcm"),
        {},
    ),
    pytest.param(
        J2KI_08_08_3_0_1F_YBR_ICT,
        get_testdata_file("US1_UNCI.dcm"),
        {},
    ),
    pytest.param(
        J2KI_16_10_1_0_1F_M1,
        get_testdata_file("RG3_UNCI.dcm"),
        {},
    ),
    pytest.param(
        J2KI_16_12_1_0_1F_M2,
        get_testdata_file("MR2_UNCI.dcm"),
        {},
    ),
    pytest.param(
        J2KI_16_15_1_0_1F_M1,
        get_testdata_file("RG1_UNCI.dcm"),
        {},
    ),
    pytest.param(
        J2KI_16_14_1_1_1F_M2,
        get_testdata_file("693_UNCI.dcm"),
        {'BitsStored': 16},
    ),
    pytest.param(
        J2KI_16_16_1_1_1F_M2,
        get_testdata_file("JPEG2000_UNC.dcm"),
        {},
    ),
]


def test_unsupported_syntaxes():
    """Test that UNSUPPORTED_SYNTAXES is as expected."""
    for syntax in SUPPORTED_SYNTAXES:
        assert syntax not in UNSUPPORTED_SYNTAXES


@pytest.mark.skipif(not HAVE_PYLIBJPEG, reason='pylibjpeg not available')
class TestHandler:
    """Tests for handling Pixel Data with the handler."""
    def setup(self):
        """Setup the test datasets and the environment."""
        self.original_handlers = pydicom.config.pixel_data_handlers
        pydicom.config.pixel_data_handlers = [NP_HANDLER, LJ_HANDLER]

    def teardown(self):
        """Restore the environment."""
        pydicom.config.pixel_data_handlers = self.original_handlers

    def test_environment(self):
        """Check that the testing environment is as expected."""
        assert HAVE_NP
        assert HAVE_PYLIBJPEG
        assert LJ_HANDLER is not None

    def test_unsupported_syntax_raises(self):
        """Test pixel_array raises exception for unsupported syntaxes."""
        pydicom.config.pixel_data_handlers = [LJ_HANDLER]

        ds = dcmread(EXPL)
        for uid in UNSUPPORTED_SYNTAXES:
            ds.file_meta.TransferSyntaxUID = uid
            with pytest.raises((NotImplementedError, RuntimeError)):
                ds.pixel_array

    @pytest.mark.skipif(HAVE_LJ and HAVE_OJ, reason="plugins available")
    def test_no_plugins_raises(self):
        """Test exception raised if required plugin missing."""
        ds = dcmread(JPGB_08_08_3_0_1F_YBR_FULL)
        msg = (
            r"Unable to convert the Pixel Data as the 'pylibjpeg-libjpeg' "
            r"plugin is not installed"
        )
        with pytest.raises(RuntimeError, match=msg):
            ds.pixel_array

        ds = dcmread(J2KI_08_08_3_0_1F_RGB)
        msg = (
            r"Unable to convert the Pixel Data as the 'pylibjpeg-openjpeg' "
            r"plugin is not installed"
        )
        with pytest.raises(RuntimeError, match=msg):
            ds.pixel_array

    def test_change_photometric_interpretation(self):
        """Test returned value."""
        ds = dcmread(J2KR_16_12_1_0_1F_M2)
        func = LJ_HANDLER.should_change_PhotometricInterpretation_to_RGB
        assert func(ds) is False


@pytest.mark.skipif(not TEST_JPEG, reason="no -libjpeg plugin")
class TestJPEG:
    def setup(self):
        """Setup the test datasets and the environment."""
        self.original_handlers = pydicom.config.pixel_data_handlers
        pydicom.config.pixel_data_handlers = [NP_HANDLER, LJ_HANDLER]

    def teardown(self):
        """Restore the environment."""
        pydicom.config.pixel_data_handlers = self.original_handlers

    @pytest.mark.parametrize('fpath, data', JPG_REFERENCE_DATA)
    def test_properties(self, fpath, data):
        """Test dataset and pixel array properties are as expected."""
        ds = dcmread(fpath)
        assert ds.file_meta.TransferSyntaxUID == data[0]
        assert ds.BitsAllocated == data[1]
        assert ds.SamplesPerPixel == data[2]
        assert ds.PixelRepresentation == data[3]
        assert getattr(ds, 'NumberOfFrames', 1) == data[4]

        arr = ds.pixel_array

        assert arr.flags.writeable
        assert data[5] == arr.shape
        assert arr.dtype == data[6]

    @pytest.mark.parametrize('fpath, rpath, val, tol', JPG_MATCHING_DATASETS)
    def test_array(self, fpath, rpath, val, tol):
        """Test pixel_array returns correct values."""
        ds = dcmread(fpath)
        arr = ds.pixel_array
        if 'YBR' in ds.PhotometricInterpretation:
            arr = convert_color_space(arr, ds.PhotometricInterpretation, 'RGB')

        ref = dcmread(rpath).pixel_array

        if val:
            assert tuple(arr[5, 50, :]) == val[0]
            assert tuple(arr[15, 50, :]) == val[1]
            assert tuple(arr[25, 50, :]) == val[2]
            assert tuple(arr[35, 50, :]) == val[3]
            assert tuple(arr[45, 50, :]) == val[4]
            assert tuple(arr[55, 50, :]) == val[5]
            assert tuple(arr[65, 50, :]) == val[6]
            assert tuple(arr[75, 50, :]) == val[7]
            assert tuple(arr[85, 50, :]) == val[8]
            assert tuple(arr[95, 50, :]) == val[9]

        # All results within `tol` intensity units of the reference
        assert np.allclose(arr, ref, atol=tol)

    @pytest.mark.parametrize('fpath, rpath, val, tol', JPG_MATCHING_DATASETS)
    def test_generate_frames(self, fpath, rpath, val, tol):
        """Test pixel_array returns correct values."""
        ds = dcmread(fpath)
        frame_generator = generate_frames(ds)
        ref = dcmread(rpath).pixel_array

        nr_frames = getattr(ds, 'NumberOfFrames', 1)
        for ii in range(nr_frames):
            arr = next(frame_generator)
            if 'YBR' in ds.PhotometricInterpretation:
                arr = convert_color_space(
                    arr, ds.PhotometricInterpretation, 'RGB'
                )

            if nr_frames > 1:
                assert np.allclose(arr, ref[ii, ...], atol=tol)
            else:
                assert np.allclose(arr, ref, atol=tol)

        with pytest.raises(StopIteration):
            next(frame_generator)

    def test_bad_file_raises(self):
        """Test a bad JPEG file raises an exception."""
        ds = dcmread(JPGE_BAD)
        msg = (
            r"libjpeg error code '-1038' returned from Decode\(\): A "
            r"misplaced marker segment was found - scan start must be zero "
            r"and scan stop must be 63 for the sequential operating modes"
        )
        with pytest.raises(RuntimeError, match=msg):
            ds.pixel_array

    def test_missing_element_raises(self):
        """Test that missing required element raises exception."""
        ds = dcmread(JPGB_08_08_3_0_1F_YBR_FULL)
        del ds.PixelData
        msg = (
            r"Unable to convert the pixel data as the following required "
            r"elements are missing from the dataset: PixelData"
        )
        with pytest.raises(AttributeError, match=msg):
            ds.pixel_array


@pytest.mark.skipif(not TEST_JPEGLS, reason="no -libjpeg plugin")
class TestJPEGLS:
    def setup(self):
        """Setup the test datasets and the environment."""
        self.original_handlers = pydicom.config.pixel_data_handlers
        pydicom.config.pixel_data_handlers = [NP_HANDLER, LJ_HANDLER]

    def teardown(self):
        """Restore the environment."""
        pydicom.config.pixel_data_handlers = self.original_handlers

    @pytest.mark.parametrize('fpath, data', JLS_REFERENCE_DATA)
    def test_properties(self, fpath, data):
        """Test dataset and pixel array properties are as expected."""
        ds = dcmread(fpath)
        assert ds.file_meta.TransferSyntaxUID == data[0]
        assert ds.BitsAllocated == data[1]
        assert ds.SamplesPerPixel == data[2]
        assert ds.PixelRepresentation == data[3]
        assert getattr(ds, 'NumberOfFrames', 1) == data[4]

        arr = ds.pixel_array

        assert arr.flags.writeable
        assert data[5] == arr.shape
        assert arr.dtype == data[6]

    def test_arrary(self):
        """Test returned array values are OK."""
        ds = dcmread(JPEG_LS_LOSSLESS)
        arr = ds.pixel_array

        # Checked against GDCM
        assert (
            [170, 193, 191, 373, 1293, 2053, 1879, 1683, 1711] ==
            arr[55:65, 35].tolist()
        )


@pytest.mark.skipif(not TEST_JPEG2K, reason="no -openjpeg plugin")
class TestJPEG2K:
    def setup(self):
        """Setup the test datasets and the environment."""
        self.original_handlers = pydicom.config.pixel_data_handlers
        pydicom.config.pixel_data_handlers = [NP_HANDLER, LJ_HANDLER]

    def teardown(self):
        """Restore the environment."""
        pydicom.config.pixel_data_handlers = self.original_handlers

    @pytest.mark.parametrize('fpath, data', J2K_REFERENCE_DATA)
    def test_properties_as_array(self, fpath, data):
        """Test dataset, pixel_array and as_array() are as expected."""
        ds = dcmread(fpath)
        assert ds.file_meta.TransferSyntaxUID == data[0]
        assert ds.BitsAllocated == data[1]
        assert ds.SamplesPerPixel == data[2]
        assert ds.PixelRepresentation == data[3]
        assert getattr(ds, 'NumberOfFrames', 1) == data[4]

        # Check Dataset.pixel_array
        arr = ds.pixel_array

        assert arr.flags.writeable
        assert data[5] == arr.shape
        assert arr.dtype == data[6]

        # Check handlers as_array() function
        arr = as_array(ds)

        assert arr.flags.writeable
        assert data[5] == arr.shape
        assert arr.dtype == data[6]

    @pytest.mark.parametrize('fpath, rpath, fixes', J2K_MATCHING_DATASETS)
    def test_array(self, fpath, rpath, fixes):
        """Test pixel_array returns correct values."""
        ds = dcmread(fpath)
        arr = ds.pixel_array

        ref = dcmread(rpath).pixel_array
        assert np.array_equal(arr, ref)

    @pytest.mark.parametrize('fpath, rpath, fixes', J2K_MATCHING_DATASETS)
    def test_generate_frames(self, fpath, rpath, fixes):
        """Test pixel_array returns correct values."""
        ds = dcmread(fpath)
        frame_generator = generate_frames(ds)
        ref = dcmread(rpath).pixel_array

        nr_frames = getattr(ds, 'NumberOfFrames', 1)
        for ii in range(nr_frames):
            arr = next(frame_generator)

            if nr_frames > 1:
                assert np.array_equal(arr, ref[ii, ...])
            else:
                assert np.array_equal(arr, ref)

        with pytest.raises(StopIteration):
            next(frame_generator)

    def test_warnings(self):
        """Test the plugin warnings work."""
        # Bits Stored
        ds = dcmread(J2KR_16_14_1_1_1F_M2)
        msg = (
            r"The \(0028,0101\) Bits Stored value '16' in the dataset does "
            r"not match the component precision value '14' found in the JPEG "
            r"2000 data. It's recommended that you change the Bits Stored "
            r"value to produce the correct output"
        )
        with pytest.warns(UserWarning, match=msg):
            ds.pixel_array

        # Pixel Representation
        ds.BitsStored = 14
        ds.PixelRepresentation = 0
        msg = (
            r"The \(0028,0103\) Pixel Representation value '0' \(unsigned\) "
            r"in the dataset does not match the format of the values found in "
            r"the JPEG 2000 data 'signed'. It's recommended that you change "
            r"the  Pixel Representation value to produce the correct output"
        )
        with pytest.warns(UserWarning, match=msg):
            ds.pixel_array

        # Samples per Pixel
        ds.PixelRepresentation = 0
        ds.SamplesPerPixel = 3
        msg = (
            r"The \(0028,0002\) Samples per Pixel value '3' in the dataset "
            r"does not match the number of components '1' found in the JPEG "
            r"2000 data. It's recommended that you change the  Samples per "
            r"Pixel value to produce the correct output"
        )
        with pytest.warns(UserWarning, match=msg):
            with pytest.raises(ValueError):
                ds.pixel_array

        # JP2 header
        ds = dcmread(J2KR_08_08_3_0_1F_YBR_RCT)
        msg = (
            r"The \(7FE0,0010\) Pixel Data contains a JPEG 2000 codestream "
            r"with the optional JP2 file format header, which is "
            r"non-conformant to the DICOM Standard \(Part 5, Annex A.4.4\)"
        )
        with pytest.warns(UserWarning, match=msg):
            ds.pixel_array

    def test_decompress_using_pylibjpeg(self):
        """Test decompressing JPEG2K with pylibjpeg handler succeeds."""
        ds = dcmread(J2KR_16_12_1_0_1F_M2)
        ds.decompress(handler_name='pylibjpeg')
        arr = ds.pixel_array

        ds = dcmread(get_testdata_file("MR2_J2KR.dcm"))
        ref = ds.pixel_array
        assert np.array_equal(arr, ref)

    def test_pixel_rep_mismatch(self):
        """Test mismatched j2k sign and Pixel Representation."""
        ds = dcmread(J2KR_16_13_1_1_1F_M2_MISMATCH)
        assert 1 == ds.PixelRepresentation
        assert 13 == ds.BitsStored

        bs = defragment_data(ds.PixelData)
        params = get_j2k_parameters(bs)
        assert 13 == params["precision"]
        assert not params["is_signed"]
        arr = ds.pixel_array

        assert 'int16' == arr.dtype
        assert (512, 512) == arr.shape
        assert arr.flags.writeable

        assert -2000 == arr[0, 0]
        assert [621, 412, 138, -193, -520, -767, -907, -966, -988, -995] == (
            arr[47:57, 279].tolist()
        )
        assert [-377, -121, 141, 383, 633, 910, 1198, 1455, 1638, 1732] == (
            arr[328:338, 106].tolist()
        )



# Copyright 2008-2020 pydicom authors. See LICENSE file for details.
"""Unit tests for pydicom.data_manager"""

import json
import os
from os.path import basename
from pathlib import Path
import shutil

import pytest

from pydicom.data import (
    get_charset_files, get_testdata_files, get_palette_files, fetch_data_files
)
from pydicom.data.data_manager import (
    DATA_ROOT, get_testdata_file, EXTERNAL_DATA_SOURCES
)
from pydicom.data import download
from pydicom.data.download import (
    get_data_dir, calculate_file_hash, get_cached_filehash
)


EXT_PYDICOM = False
if 'pydicom-data' in EXTERNAL_DATA_SOURCES:
    EXT_PYDICOM = True


@pytest.fixture
def download_failure():
    """Simulate a download failure."""
    download._SIMULATE_NETWORK_OUTAGE = True
    yield
    download._SIMULATE_NETWORK_OUTAGE = False


class TestGetData:
    def test_get_dataset(self):
        """Test the different functions to get lists of data files."""
        # The cached files downloaded from the pydicom-data repo
        cached_data_test_files = str(get_data_dir())

        # If pydicom-data is available locally
        ext_path = None
        if 'pydicom-data' in EXTERNAL_DATA_SOURCES:
            ext_path = os.fspath(
                EXTERNAL_DATA_SOURCES['pydicom-data'].data_path
            )

        # Test base locations
        charbase = os.path.join(DATA_ROOT, 'charset_files')
        assert os.path.exists(charbase)

        testbase = os.path.join(DATA_ROOT, 'test_files')
        assert os.path.exists(testbase)

        # Test file get
        chardata = get_charset_files()
        assert 15 < len(chardata)

        # Test that top level file is included
        bases = [basename(x) for x in chardata]

        # Test that subdirectory files included
        testdata = get_testdata_files()
        bases = [basename(x) for x in testdata]
        assert '2693' in bases
        assert 70 < len(testdata)

        # The files should be from their respective bases
        for x in testdata:
            # Don't check files from external sources other than pydicom-data
            if (
                testbase not in x
                and cached_data_test_files not in x
                and (ext_path not in x if ext_path else True)
            ):
                continue

            assert (
                testbase in x
                or cached_data_test_files in x
                or (ext_path in x if ext_path else False)
            )

        for x in chardata:
            assert charbase in x

    def test_get_dataset_pattern(self):
        """Test that pattern is working properly."""
        pattern = 'CT_small*'
        filename = get_testdata_files(pattern)
        assert filename[0].endswith('CT_small.dcm')

        pattern = 'chrX1*'
        filename = get_charset_files(pattern)
        assert filename[0].endswith('chrX1.dcm')

    def test_get_testdata_file(self):
        """Test that file name is working properly."""
        p = Path(get_testdata_file('DICOMDIR'))
        assert "DICOMDIR" == p.name.upper()

    def test_get_palette_files(self):
        """Test data_manager.get_palette_files."""
        palbase = os.path.join(DATA_ROOT, 'palettes')
        assert os.path.exists(palbase)

        palettes = get_palette_files('*.dcm')
        assert 8 == len(palettes)

        for x in palettes:
            assert palbase in x


@pytest.mark.skipif(not EXT_PYDICOM, reason="pydicom-data not installed")
class TestExternalDataSource:
    """Tests for the external data sources."""
    def setup(self):
        self.dpath = EXTERNAL_DATA_SOURCES["pydicom-data"].data_path

        # Backup the 693_UNCI.dcm file
        p = self.dpath / "693_UNCI.dcm"
        shutil.copy(p, self.dpath / "PYTEST_BACKUP")

    def teardown(self):
        # Restore the backed-up file
        p = self.dpath / "693_UNCI.dcm"
        shutil.copy(self.dpath / "PYTEST_BACKUP", p)
        os.remove(self.dpath / "PYTEST_BACKUP")

        if 'mylib' in EXTERNAL_DATA_SOURCES:
            del EXTERNAL_DATA_SOURCES['mylib']

    def as_posix(self, path):
        """Return `path` as a posix path"""
        return Path(path).as_posix()

    def test_get_testdata_file_local(self):
        """Test that local data path retrieved OK."""
        fname = "CT_small.dcm"
        fpath = self.as_posix(get_testdata_file(fname))
        assert "pydicom/data/test_files" in fpath

    def test_get_testdata_file_external(self):
        """Test that external data source preferred over cache."""
        fname = "693_UNCI.dcm"
        fpath = self.as_posix(get_testdata_file(fname))
        assert "data_store/data" in fpath

    def test_get_testdata_file_external_hash_mismatch(self):
        """Test that the external source is not used when hash is not OK."""
        p = self.dpath / "693_UNCI.dcm"
        with open(p, 'wb') as f:
            f.write(b"\x00\x01")

        ext_hash = calculate_file_hash(p)
        ref_hash = get_cached_filehash(p.name)
        assert ext_hash != ref_hash
        fpath = self.as_posix(get_testdata_file(p.name))
        assert ".pydicom/data" in fpath

    def test_get_testdata_file_external_hash_match(self):
        """Test that external source is used when hash is OK."""
        fname = "693_UNCI.dcm"
        p = self.dpath / fname
        ext_hash = calculate_file_hash(p)
        ref_hash = get_cached_filehash(p.name)
        assert ext_hash == ref_hash
        fpath = self.as_posix(get_testdata_file(fname))
        assert "data_store/data" in fpath

    def test_get_testdata_file_external_ignore_hash(self):
        """Test that non-pydicom-data external source ignores hash check."""
        EXTERNAL_DATA_SOURCES['mylib'] = EXTERNAL_DATA_SOURCES['pydicom-data']
        p = self.dpath / "693_UNCI.dcm"
        with open(p, 'wb') as f:
            f.write(b"\x00\x01")

        ext_hash = calculate_file_hash(p)
        ref_hash = get_cached_filehash(p.name)
        assert ext_hash != ref_hash
        fpath = self.as_posix(get_testdata_file(p.name))
        assert "data_store/data" in fpath

    def test_get_testdata_file_missing(self):
        """Test no such file available."""
        fname = "MY_MISSING_FILE.dcm"
        assert get_testdata_file(fname) is None

    def test_get_testdata_files_local(self):
        """Test that local data paths retrieved OK."""
        fname = "CT_small*"
        paths = get_testdata_files(fname)
        assert 1 == len(paths)
        assert "pydicom/data/test_files" in self.as_posix(paths[0])

    def test_get_testdata_files_local_external_and_cache(self):
        """Test that local, external and cache paths retrieved OK."""
        fname = "693*"
        paths = get_testdata_files(fname)
        assert 7 == len(paths)
        # Local preferred first
        assert "pydicom/data/test_files" in self.as_posix(paths[0])
        # External source preferred second
        assert "data_store/data" in self.as_posix(paths[1])
        # Cache source preferred last
        assert ".pydicom/data" in self.as_posix(paths[4])

    def test_get_testdata_files_hash_match(self):
        """Test that the external source is not used when hash is not OK."""
        p = self.dpath / "693_UNCI.dcm"
        ext_hash = calculate_file_hash(p)
        ref_hash = get_cached_filehash(p.name)
        assert ext_hash == ref_hash
        fpaths = get_testdata_files("693_UNCI*")
        fpaths = [self.as_posix(p) for p in fpaths]
        assert 2 == len(fpaths)
        assert "data_store/data" in fpaths[0]
        assert ".pydicom/data" in fpaths[1]

    def test_get_testdata_files_hash_mismatch(self):
        """Test that the external source is not used when hash is not OK."""
        p = self.dpath / "693_UNCI.dcm"
        with open(p, 'wb') as f:
            f.write(b"\x00\x01")

        ext_hash = calculate_file_hash(p)
        ref_hash = get_cached_filehash(p.name)
        assert ext_hash != ref_hash
        fpaths = get_testdata_files("693_UNCI*")
        fpaths = [self.as_posix(p) for p in fpaths]
        assert 1 == len(fpaths)
        assert ".pydicom/data" in fpaths[0]

    def test_get_testdata_files_external_ignore_hash(self):
        """Test that non-pydicom-data external source ignores hash check."""
        EXTERNAL_DATA_SOURCES['mylib'] = EXTERNAL_DATA_SOURCES['pydicom-data']
        p = self.dpath / "693_UNCI.dcm"
        with open(p, 'wb') as f:
            f.write(b"\x00\x01")

        ext_hash = calculate_file_hash(p)
        ref_hash = get_cached_filehash(p.name)
        assert ext_hash != ref_hash
        fpaths = get_testdata_files("693_UNCI*")
        fpaths = [self.as_posix(p) for p in fpaths]
        assert 2 == len(fpaths)
        assert "data_store/data" in fpaths[0]
        assert ".pydicom/data" in fpaths[1]


@pytest.mark.skipif(EXT_PYDICOM, reason="pydicom-data installed")
class TestDownload:
    """Tests for the download module."""
    def test_get_testdata_file_network_outage(self, download_failure):
        """Test a network outage when using get_testdata_file."""
        fname = "693_UNCI.dcm"
        msg = (
            r"A download failure occurred while attempting to "
            r"retrieve 693_UNCI.dcm"
        )
        with pytest.warns(UserWarning, match=msg):
            assert get_testdata_file(fname) is None

    def test_get_testdata_files_network_outage(self, download_failure):
        """Test a network outage when using get_testdata_files."""
        msg = (
            r"One or more download failures occurred, the list of matching "
            r"file paths may be incomplete"
        )
        with pytest.warns(UserWarning, match=msg):
            assert [] == get_testdata_files("693_UN*")


def test_fetch_data_files():
    """Test fetch_data_files()."""
    # Remove a single file from the cache
    cache = get_data_dir()
    path = cache / "693_J2KR.dcm"
    if path.exists():
        path.unlink()

    assert not path.exists()
    fetch_data_files()
    assert path.exists()


def test_fetch_data_files_download_failure(download_failure):
    """Test fetch_data_files() with download failures."""
    msg = r"An error occurred downloading the following files:"
    with pytest.raises(RuntimeError, match=msg):
        fetch_data_files()


def test_hashes():
    """Test for duplicates in hashes.json."""
    # We can't have case mixes because windows filenames are case insensitive
    root = Path(DATA_ROOT)
    with open(root / "hashes.json", "r") as f:
        filenames = json.load(f).keys()
        filenames = [name.lower() for name in filenames]
        assert len(set(filenames)) == len(filenames)


def test_urls():
    """Test for duplicates in urls.json."""
    # We can't have case mixes because windows filenames are case insensitive
    root = Path(DATA_ROOT)
    with open(root / "urls.json", "r") as f:
        filenames = json.load(f).keys()
        filenames = [name.lower() for name in filenames]
        assert len(set(filenames)) == len(filenames)



# Copyright 2008-2018 pydicom authors. See LICENSE file for details.
"""Benchmarks for the numpy_handler module.

Requires asv and numpy.
"""

from platform import python_implementation
from tempfile import TemporaryFile

import numpy as np

from pydicom import dcmread
from pydicom.data import get_testdata_file
from pydicom.dataset import Dataset, FileMetaDataset
from pydicom.pixel_data_handlers.numpy_handler import get_pixeldata
from pydicom.uid import ExplicitVRLittleEndian, generate_uid

# 1/1, 1 sample/pixel, 1 frame
EXPL_1_1_1F = get_testdata_file("liver_1frame.dcm")
# 1/1, 1 sample/pixel, 3 frame
EXPL_1_1_3F = get_testdata_file("liver.dcm")
# 8/8, 1 sample/pixel, 1 frame
EXPL_8_1_1F = get_testdata_file("OBXXXX1A.dcm")
# 8/8, 1 sample/pixel, 2 frame
EXPL_8_1_2F = get_testdata_file("OBXXXX1A_2frame.dcm")
# 8/8, 3 sample/pixel, 1 frame
EXPL_8_3_1F = get_testdata_file("SC_rgb.dcm")
# 8/8, 3 sample/pixel, 1 frame, YBR_FULL_422
EXPL_8_3_1F_YBR422 = get_testdata_file('SC_ybr_full_422_uncompressed.dcm')
# 8/8, 3 sample/pixel, 2 frame
EXPL_8_3_2F = get_testdata_file("SC_rgb_2frame.dcm")
# 16/16, 1 sample/pixel, 1 frame
EXPL_16_1_1F = get_testdata_file("MR_small.dcm")
# 16/12, 1 sample/pixel, 10 frame
EXPL_16_1_10F = get_testdata_file("emri_small.dcm")
# 16/16, 3 sample/pixel, 1 frame
EXPL_16_3_1F = get_testdata_file("SC_rgb_16bit.dcm")
# 16/16, 3 sample/pixel, 2 frame
EXPL_16_3_2F = get_testdata_file("SC_rgb_16bit_2frame.dcm")
# 32/32, 1 sample/pixel, 1 frame
IMPL_32_1_1F = get_testdata_file("rtdose_1frame.dcm")
# 32/32, 1 sample/pixel, 15 frame
IMPL_32_1_15F = get_testdata_file("rtdose.dcm")
# 32/32, 3 sample/pixel, 1 frame
EXPL_32_3_1F = get_testdata_file("SC_rgb_32bit.dcm")
# 32/32, 3 sample/pixel, 2 frame
EXPL_32_3_2F = get_testdata_file("SC_rgb_32bit_2frame.dcm")


def _create_temporary_dataset(shape=(100, 1024, 1024, 3), bit_depth=16):
    """Function to create a temporary dataset for use in testing.

    Parameters
    ----------
    shape : 4-tuple
        The (frames, rows, columns, channels) of the test dataset.
    bit_depth : int
        The BitsAllocated value to use for the dataset, one of 8, 16, 32, 64.

    Returns
    -------
    tempfile.TemporaryFile
        A created DICOM File Format conformant dataset.
    """
    ds = Dataset()
    ds.is_little_endian = True
    ds.is_implicit_VR = False
    ds.file_meta = FileMetaDataset()
    ds.file_meta.TransferSyntaxUID = ExplicitVRLittleEndian
    ds.SOPClassUID = '1.2.3.4'
    ds.SOPInstanceUID = generate_uid()
    ds.BitsAllocated = bit_depth
    ds.PixelRepresentation = 0
    ds.PlanarConfiguration = 0
    ds.Rows = shape[1]
    ds.Columns = shape[2]
    ds.NumberOfFrames = shape[0]
    ds.SamplesPerPixel = shape[3]
    if shape[3] == 1:
        ds.PhotometricInterpretation = 'MONOCHROME2'
    elif shape[3] == 3:
        ds.PhotometricInterpretation = 'RGB'

    arr = np.zeros(shape, dtype='uint{}'.format(bit_depth))
    ds.PixelData = arr.tobytes()

    if len(ds.PixelData) % 2:
        ds.PixelData += b'\x00'

    tfile = TemporaryFile(mode='w+b')
    ds.save_as(tfile, write_like_original=False)
    tfile.seek(0)

    return tfile


class TimeGetPixelData_LargeDataset:
    """Time tests for numpy_handler.get_pixeldata with large datasets."""
    def setup(self):
        """Setup the tests."""
        self.no_runs = 100

        self.ds_16_3_100 = dcmread(_create_temporary_dataset())

    def time_large_dataset(self):
        """Time reading pixel data from a large dataset."""
        for ii in range(self.no_runs):
            get_pixeldata(self.ds_16_3_100)


class TimeGetPixelData:
    """Time tests for numpy_handler.get_pixeldata."""
    def setup(self):
        """Setup the tests."""
        self.no_runs = 100

        self.ds_1_1_1 = dcmread(EXPL_1_1_1F)
        self.ds_1_1_3 = dcmread(EXPL_1_1_3F)
        self.ds_8_1_1 = dcmread(EXPL_8_1_1F)
        self.ds_8_1_2 = dcmread(EXPL_8_1_2F)
        self.ds_8_3_1 = dcmread(EXPL_8_3_1F)
        self.ds_8_3_2 = dcmread(EXPL_8_3_2F)
        self.ds_16_1_1 = dcmread(EXPL_16_1_1F)
        self.ds_16_1_10 = dcmread(EXPL_16_1_10F)
        self.ds_16_3_1 = dcmread(EXPL_16_3_1F)
        self.ds_16_3_2 = dcmread(EXPL_16_3_2F)
        self.ds_32_1_1 = dcmread(IMPL_32_1_1F)
        self.ds_32_1_15 = dcmread(IMPL_32_1_15F)
        self.ds_32_3_1 = dcmread(EXPL_32_3_1F)
        self.ds_32_3_2 = dcmread(EXPL_32_3_2F)
        self.ds_ybr_422 = dcmread(EXPL_8_3_1F_YBR422)

    def time_1bit_1sample_1frame(self):
        """Time retrieval of 1-bit, 1 sample/pixel, 1 frame."""
        no_runs = self.no_runs
        if 'PyPy' in python_implementation():
            no_runs = 1

        for ii in range(no_runs):
            get_pixeldata(self.ds_1_1_1)

    def time_1bit_1sample_3frame(self):
        """Time retrieval of 1-bit, 1 sample/pixel, 3 frame."""
        no_runs = self.no_runs
        if 'PyPy' in python_implementation():
            no_runs = 1

        for ii in range(no_runs):
            get_pixeldata(self.ds_1_1_3)

    def time_8bit_1sample_1frame(self):
        """Time retrieval of 8-bit, 1 sample/pixel, 1 frame."""
        for ii in range(self.no_runs):
            get_pixeldata(self.ds_8_1_1)

    def time_8bit_1sample_2frame(self):
        """Time retrieval of 8-bit, 1 sample/pixel, 2 frame."""
        for ii in range(self.no_runs):
            get_pixeldata(self.ds_8_1_2)

    def time_8bit_3sample_1frame(self):
        """Time retrieval of 8-bit, 3 sample/pixel, 1 frame."""
        for ii in range(self.no_runs):
            get_pixeldata(self.ds_8_3_1)

    def time_8bit_3sample_2frame(self):
        """Time retrieval of 8-bit, 3 sample/pixel, 2 frame."""
        for ii in range(self.no_runs):
            get_pixeldata(self.ds_8_3_2)

    def time_16bit_1sample_1frame(self):
        """Time retrieval of 16-bit, 1 sample/pixel, 1 frame."""
        for ii in range(self.no_runs):
            get_pixeldata(self.ds_16_1_1)

    def time_16bit_1sample_10frame(self):
        """Time retrieval of 16-bit, 1 sample/pixel, 10 frame."""
        for ii in range(self.no_runs):
            get_pixeldata(self.ds_16_1_10)

    def time_16bit_3sample_1frame(self):
        """Time retrieval of 16-bit, 3 sample/pixel, 1 frame."""
        for ii in range(self.no_runs):
            get_pixeldata(self.ds_16_3_1)

    def time_16bit_3sample_2frame(self):
        """Time retrieval of 16-bit, 3 sample/pixel, 2 frame."""
        for ii in range(self.no_runs):
            get_pixeldata(self.ds_16_3_2)

    def time_32bit_1sample_1frame(self):
        """Time retrieval of 32-bit, 1 sample/pixel, 1 frame."""
        for ii in range(self.no_runs):
            get_pixeldata(self.ds_32_1_1)

    def time_32bit_1sample_15frame(self):
        """Time retrieval of 32-bit, 1 sample/pixel, 15 frame."""
        for ii in range(self.no_runs):
            get_pixeldata(self.ds_32_1_15)

    def time_32bit_3sample_1frame(self):
        """Time retrieval of 32-bit, 3 sample/pixel, 1 frame."""
        for ii in range(self.no_runs):
            get_pixeldata(self.ds_32_3_1)

    def time_32bit_3sample_2frame(self):
        """Time retrieval of 32-bit, 3 sample/pixel, 2 frame."""
        for ii in range(self.no_runs):
            get_pixeldata(self.ds_32_3_2)

    def time_ybr_422(self):
        """Time retrieval of YBR_FULL_422 data."""
        for ii in range(self.no_runs):
            get_pixeldata(self.ds_ybr_422)



# Copyright 2008-2020 pydicom authors. See LICENSE file for details.
"""Test suite for util functions"""
from io import BytesIO
from pathlib import Path

import pytest

from pydicom.fileutil import path_from_pathlike


class PathLike:
    """Minimal example for path-like object"""
    def __init__(self, path: str):
        self.path = path

    def __fspath__(self):
        return self.path


class TestPathFromPathLike:
    """Test the fileutil module"""

    def test_non_pathlike_is_returned_unaltered(self):
        assert 'test.dcm' == path_from_pathlike('test.dcm')
        assert path_from_pathlike(None) is None
        file_like = BytesIO()
        assert file_like == path_from_pathlike(file_like)
        assert 42 == path_from_pathlike(42)

    def test_pathlib_path(self):
        assert 'test.dcm' == path_from_pathlike(Path('test.dcm'))

    def test_path_like(self):
        assert 'test.dcm' == path_from_pathlike(PathLike('test.dcm'))



# Copyright 2008-2019 pydicom authors. See LICENSE file for details.
"""Methods for converting Datasets and DataElements to/from json"""

import base64
from inspect import signature
import inspect
from typing import Callable, Optional, Union
import warnings

from pydicom.tag import BaseTag

# Order of keys is significant!
JSON_VALUE_KEYS = ('Value', 'BulkDataURI', 'InlineBinary',)

BINARY_VR_VALUES = ['OW', 'OB', 'OD', 'OF', 'OL', 'UN',
                    'OB or OW', 'US or OW', 'US or SS or OW']
VRs_TO_BE_FLOATS = ['DS', 'FL', 'FD', ]
VRs_TO_BE_INTS = ['IS', 'SL', 'SS', 'UL', 'US', 'US or SS']


def convert_to_python_number(value, vr):
    """Makes sure that values are either ints or floats
    based on their value representation.

    .. versionadded:: 1.4

    Parameters
    ----------
    value: Union[Union[str, int, float], List[Union[str, int, float]]]
        value of data element
    vr: str
        value representation of data element

    Returns
    -------
    Union[Union[str, int, float], List[Union[str, int, float]]]

    """
    if value is None:
        return None
    number_type = None
    if vr in VRs_TO_BE_INTS:
        number_type = int
    if vr in VRs_TO_BE_FLOATS:
        number_type = float
    if number_type is not None:
        if isinstance(value, (list, tuple,)):
            value = [number_type(e) for e in value]
        else:
            value = number_type(value)
    return value


class JsonDataElementConverter:
    """Handles conversion between JSON struct and :class:`DataElement`.

    .. versionadded:: 1.4
    """

    def __init__(
        self,
        dataset_class,
        tag,
        vr,
        value,
        value_key,
        bulk_data_uri_handler: Optional[
            Union[
                Callable[[BaseTag, str, str], object],
                Callable[[str], object]
            ]
        ] = None
    ):
        """Create a new converter instance.

        Parameters
        ----------
        dataset_class : dataset.Dataset derived class
            Class used to create sequence items.
        tag : BaseTag
            The data element tag or int.
        vr : str
            The data element value representation.
        value : list
            The data element's value(s).
        value_key : str or None
            Key of the data element that contains the value
            (options: ``{"Value", "InlineBinary", "BulkDataURI"}``)
        bulk_data_uri_handler: callable or None
            Callable function that accepts either the tag, vr and "BulkDataURI"
            or just the "BulkDataURI" of the JSON
            representation of a data element and returns the actual value of
            that data element (retrieved via DICOMweb WADO-RS)
        """
        self.dataset_class = dataset_class
        self.tag = tag
        self.vr = vr
        self.value = value
        self.value_key = value_key
        if (
            bulk_data_uri_handler and
            len(signature(bulk_data_uri_handler).parameters) == 1
        ):
            def wrapped_bulk_data_handler(tag, vr, value):
                return bulk_data_uri_handler(value)
            self.bulk_data_element_handler = wrapped_bulk_data_handler
        else:
            self.bulk_data_element_handler = bulk_data_uri_handler

    def get_element_values(self):
        """Return a the data element value or list of values.

        Returns
        -------
        str or bytes or int or float or dataset_class
        or PersonName or list of any of these types
            The value or value list of the newly created data element.
        """
        from pydicom.dataelem import empty_value_for_VR
        if self.value_key == 'Value':
            if not isinstance(self.value, list):
                fmt = '"{}" of data element "{}" must be a list.'
                raise TypeError(fmt.format(self.value_key, self.tag))
            if not self.value:
                return empty_value_for_VR(self.vr)
            element_value = [self.get_regular_element_value(v)
                             for v in self.value]
            if len(element_value) == 1 and self.vr != 'SQ':
                element_value = element_value[0]
            return convert_to_python_number(element_value, self.vr)

        # The value for "InlineBinary" shall be encoded as a base64 encoded
        # string, as shown in PS3.18, Table F.3.1-1, but the example in
        # PS3.18, Annex F.4 shows the string enclosed in a list.
        # We support both variants, as the standard is ambiguous here,
        # and do the same for "BulkDataURI".
        value = self.value
        if isinstance(value, list):
            value = value[0]

        if self.value_key == 'InlineBinary':
            if not isinstance(value, (str, bytes)):
                fmt = '"{}" of data element "{}" must be a bytes-like object.'
                raise TypeError(fmt.format(self.value_key, self.tag))
            return base64.b64decode(value)

        if self.value_key == 'BulkDataURI':
            if not isinstance(value, str):
                fmt = '"{}" of data element "{}" must be a string.'
                raise TypeError(fmt.format(self.value_key, self.tag))
            if self.bulk_data_element_handler is None:
                warnings.warn(
                    'no bulk data URI handler provided for retrieval '
                    'of value of data element "{}"'.format(self.tag)
                )
                return empty_value_for_VR(self.vr, raw=True)
            return self.bulk_data_element_handler(self.tag, self.vr, value)
        return empty_value_for_VR(self.vr)

    def get_regular_element_value(self, value):
        """Return a the data element value created from a json "Value" entry.

        Parameters
        ----------
        value : str or int or float or dict
            The data element's value from the json entry.

        Returns
        -------
        dataset_class or PersonName
        or str or int or float
            A single value of the corresponding :class:`DataElement`.
        """
        if self.vr == 'SQ':
            return self.get_sequence_item(value)

        if self.vr == 'PN':
            return self.get_pn_element_value(value)

        if self.vr == 'AT':
            try:
                return int(value, 16)
            except ValueError:
                warnings.warn('Invalid value "{}" for AT element - '
                              'ignoring it'.format(value))
            return
        return value

    def get_sequence_item(self, value):
        """Return a sequence item for the JSON dict `value`.

        Parameters
        ----------
        value : dict or None
            The sequence item from the JSON entry.

        Returns
        -------
        dataset_class
            The decoded dataset item.

        Raises
        ------
        KeyError
            If the "vr" key is missing for a contained element
        """
        ds = self.dataset_class()
        if value:
            for key, val in value.items():
                if 'vr' not in val:
                    fmt = 'Data element "{}" must have key "vr".'
                    raise KeyError(fmt.format(self.tag))
                vr = val['vr']
                unique_value_keys = tuple(
                    set(val.keys()) & set(JSON_VALUE_KEYS)
                )
                from pydicom import DataElement
                from pydicom.dataelem import empty_value_for_VR
                if not unique_value_keys:
                    # data element with no value
                    elem = DataElement(
                        tag=int(key, 16),
                        value=empty_value_for_VR(vr),
                        VR=vr)
                else:
                    value_key = unique_value_keys[0]
                    elem = DataElement.from_json(
                        self.dataset_class, key, vr,
                        val[value_key], value_key,
                        self.bulk_data_element_handler
                    )
                ds.add(elem)
        return ds

    def get_pn_element_value(self, value):
        """Return PersonName value from JSON value.

        Values with VR PN have a special JSON encoding, see the DICOM Standard,
        Part 18, :dcm:`Annex F.2.2<part18/sect_F.2.2.html>`.

        Parameters
        ----------
        value : dict
            The person name components in the JSON entry.

        Returns
        -------
        PersonName or str
            The decoded PersonName object or an empty string.
        """
        if not isinstance(value, dict):
            # Some DICOMweb services get this wrong, so we
            # workaround the issue and warn the user
            # rather than raising an error.
            warnings.warn(
                'value of data element "{}" with VR Person Name (PN) '
                'is not formatted correctly'.format(self.tag)
            )
            return value
        else:
            if 'Phonetic' in value:
                comps = ['', '', '']
            elif 'Ideographic' in value:
                comps = ['', '']
            else:
                comps = ['']
            if 'Alphabetic' in value:
                comps[0] = value['Alphabetic']
            if 'Ideographic' in value:
                comps[1] = value['Ideographic']
            if 'Phonetic' in value:
                comps[2] = value['Phonetic']
            elem_value = '='.join(comps)
            return elem_value



# Copyright 2008-2018 pydicom authors. See LICENSE file for details.
"""Test for datadict.py"""

import pytest

from pydicom import DataElement
from pydicom.dataset import Dataset
from pydicom.datadict import (keyword_for_tag, dictionary_description,
                              dictionary_has_tag, repeater_has_tag,
                              repeater_has_keyword, get_private_entry,
                              dictionary_VM, private_dictionary_VR,
                              private_dictionary_VM, add_private_dict_entries,
                              add_private_dict_entry)
from pydicom.datadict import add_dict_entry, add_dict_entries


class TestDict:
    def test_tag_not_found(self):
        """dicom_dictionary: CleanName returns blank string for unknown tag"""
        assert '' == keyword_for_tag(0x99991111)

    def test_repeaters(self):
        """dicom_dictionary: Tags with "x" return correct dict info........"""
        assert 'Transform Label' == dictionary_description(0x280400)
        assert ('Rows For Nth Order Coefficients' ==
                dictionary_description(0x280410))

    def test_dict_has_tag(self):
        """Test dictionary_has_tag"""
        assert dictionary_has_tag(0x00100010)
        assert not dictionary_has_tag(0x11110010)

    def test_repeater_has_tag(self):
        """Test repeater_has_tag"""
        assert repeater_has_tag(0x60000010)
        assert repeater_has_tag(0x60020010)
        assert not repeater_has_tag(0x00100010)

    def test_repeater_has_keyword(self):
        """Test repeater_has_keyword"""
        assert repeater_has_keyword('OverlayData')
        assert not repeater_has_keyword('PixelData')

    def test_get_private_entry(self):
        """Test get_private_entry"""
        # existing entry
        entry = get_private_entry((0x0903, 0x0011), 'GEIIS PACS')
        assert 'US' == entry[0]  # VR
        assert '1' == entry[1]  # VM
        assert 'Significant Flag' == entry[2]  # name
        assert not entry[3]  # is retired

        # existing entry in another slot
        entry = get_private_entry((0x0903, 0x1011), 'GEIIS PACS')
        assert 'Significant Flag' == entry[2]  # name

        # non-existing entry
        with pytest.raises(KeyError):
            get_private_entry((0x0903, 0x0011), 'Nonexisting')
        with pytest.raises(KeyError):
            get_private_entry((0x0903, 0x0091), 'GEIIS PACS')

    def test_add_entry(self):
        """dicom_dictionary: Can add and use a single dictionary entry"""
        add_dict_entry(0x10021001, "UL", "TestOne", "Test One")
        add_dict_entry(0x10021002, "DS", "TestTwo", "Test Two", VM='3')
        ds = Dataset()
        ds.TestOne = 'test'
        ds.TestTwo = ['1', '2', '3']

    def test_add_entry_raises_for_private_tag(self):
        with pytest.raises(ValueError,
                           match='Private tags cannot be '
                                 'added using "add_dict_entries"'):
            add_dict_entry(0x10011101, 'DS', 'Test One', 'Test One')

    def test_add_entries(self):
        """dicom_dictionary: add and use a dict of new dictionary entries"""
        new_dict_items = {
            0x10021001: ('UL', '1', "Test One", '', 'TestOne'),
            0x10021002: ('DS', '3', "Test Two", '', 'TestTwo'),
        }
        add_dict_entries(new_dict_items)
        ds = Dataset()
        ds.TestOne = 'test'
        ds.TestTwo = ['1', '2', '3']

    def test_add_entries_raises_for_private_tags(self):
        new_dict_items = {
            0x10021001: ('UL', '1', 'Test One', '', 'TestOne'),
            0x10011002: ('DS', '3', 'Test Two', '', 'TestTwo'),
        }
        with pytest.raises(ValueError, match='Private tags cannot be added '
                                             'using "add_dict_entries"'):
            add_dict_entries(new_dict_items)

    def test_add_private_entry(self):
        add_private_dict_entry('ACME 3.1', 0x10011101, 'DS', 'Test One', '3')
        entry = get_private_entry((0x1001, 0x0001), 'ACME 3.1')
        assert 'DS' == entry[0]  # VR
        assert '3' == entry[1]  # VM
        assert 'Test One' == entry[2]  # description

    def test_add_private_entry_raises_for_non_private_tag(self):
        msg = (
            r"Non-private tags cannot be added using "
            r"'add_private_dict_entries\(\)' \- use "
            r"'add_dict_entries\(\)' instead"
        )
        with pytest.raises(ValueError, match=msg):
            add_private_dict_entry('ACME 3.1', 0x10021101, 'DS', 'Test One')

    def test_add_private_entries(self):
        """dicom_dictionary: add and use a dict of new dictionary entries"""
        new_dict_items = {
            0x10011001: ('SH', '1', "Test One",),
            0x10011002: ('DS', '3', "Test Two", '', 'TestTwo'),
        }
        add_private_dict_entries('ACME 3.1', new_dict_items)
        ds = Dataset()
        ds[0x10010010] = DataElement(0x10010010, 'LO', 'ACME 3.1')
        ds[0x10011001] = DataElement(0x10011001, 'SH', 'Test')
        ds[0x10011002] = DataElement(0x10011002, 'DS', '1\\2\\3')

        assert 'Test' == ds[0x10011001].value
        assert [1, 2, 3] == ds[0x10011002].value

    def test_add_private_entries_raises_for_non_private_tags(self):
        new_dict_items = {
            0x10021001: ('UL', '1', 'Test One', '', 'TestOne'),
            0x10011002: ('DS', '3', 'Test Two', '', 'TestTwo'),
        }
        msg = (
            r"Non-private tags cannot be added using "
            r"'add_private_dict_entries\(\)' \- use "
            r"'add_dict_entries\(\)' instead"
        )
        with pytest.raises(ValueError, match=msg):
            add_private_dict_entries('ACME 3.1', new_dict_items)

    def test_dictionary_VM(self):
        """Test dictionary_VM"""
        assert dictionary_VM(0x00000000) == '1'
        assert dictionary_VM(0x00081163) == '2'
        assert dictionary_VM(0x0000901) == '1-n'
        assert dictionary_VM(0x00041141) == '1-8'
        assert dictionary_VM(0x00080008) == '2-n'
        assert dictionary_VM(0x00080309) == '1-3'
        assert dictionary_VM(0x00081162) == '3-3n'

    def test_private_dict_VR(self):
        """Test private_dictionary_VR"""
        assert private_dictionary_VR(0x00090000, 'ACUSON') == 'IS'

    def test_private_dict_VM(self):
        """Test private_dictionary_VM"""
        assert private_dictionary_VM(0x00090000, 'ACUSON') == '1'



"""
Storage SOP Class UIDs auto-generated by generate_storage_sopclass_uids.py
"""

from pydicom.uid import UID

MediaStorageDirectoryStorage = UID(
    '1.2.840.10008.1.3.10')
ComputedRadiographyImageStorage = UID(
    '1.2.840.10008.5.1.4.1.1.1')
DigitalXRayImageStorageForPresentation = UID(
    '1.2.840.10008.5.1.4.1.1.1.1')
DigitalXRayImageStorageForProcessing = UID(
    '1.2.840.10008.5.1.4.1.1.1.1.1')
DigitalMammographyXRayImageStorageForPresentation = UID(
    '1.2.840.10008.5.1.4.1.1.1.2')
DigitalMammographyXRayImageStorageForProcessing = UID(
    '1.2.840.10008.5.1.4.1.1.1.2.1')
DigitalIntraOralXRayImageStorageForPresentation = UID(
    '1.2.840.10008.5.1.4.1.1.1.3')
DigitalIntraOralXRayImageStorageForProcessing = UID(
    '1.2.840.10008.5.1.4.1.1.1.3.1')
EncapsulatedPDFStorage = UID(
    '1.2.840.10008.5.1.4.1.1.104.1')
EncapsulatedCDAStorage = UID(
    '1.2.840.10008.5.1.4.1.1.104.2')
EncapsulatedSTLStorage = UID(
    '1.2.840.10008.5.1.4.1.1.104.3')
EncapsulatedOBJStorage = UID(
    '1.2.840.10008.5.1.4.1.1.104.4')
EncapsulatedMTLStorage = UID(
    '1.2.840.10008.5.1.4.1.1.104.5')
GrayscaleSoftcopyPresentationStateStorage = UID(
    '1.2.840.10008.5.1.4.1.1.11.1')
SegmentedVolumeRenderingVolumetricPresentationStateStorage = UID(
    '1.2.840.10008.5.1.4.1.1.11.10')
MultipleVolumeRenderingVolumetricPresentationStateStorage = UID(
    '1.2.840.10008.5.1.4.1.1.11.11')
ColorSoftcopyPresentationStateStorage = UID(
    '1.2.840.10008.5.1.4.1.1.11.2')
PseudoColorSoftcopyPresentationStateStorage = UID(
    '1.2.840.10008.5.1.4.1.1.11.3')
BlendingSoftcopyPresentationStateStorage = UID(
    '1.2.840.10008.5.1.4.1.1.11.4')
XAXRFGrayscaleSoftcopyPresentationStateStorage = UID(
    '1.2.840.10008.5.1.4.1.1.11.5')
GrayscalePlanarMPRVolumetricPresentationStateStorage = UID(
    '1.2.840.10008.5.1.4.1.1.11.6')
CompositingPlanarMPRVolumetricPresentationStateStorage = UID(
    '1.2.840.10008.5.1.4.1.1.11.7')
AdvancedBlendingPresentationStateStorage = UID(
    '1.2.840.10008.5.1.4.1.1.11.8')
VolumeRenderingVolumetricPresentationStateStorage = UID(
    '1.2.840.10008.5.1.4.1.1.11.9')
XRayAngiographicImageStorage = UID(
    '1.2.840.10008.5.1.4.1.1.12.1')
EnhancedXAImageStorage = UID(
    '1.2.840.10008.5.1.4.1.1.12.1.1')
XRayRadiofluoroscopicImageStorage = UID(
    '1.2.840.10008.5.1.4.1.1.12.2')
EnhancedXRFImageStorage = UID(
    '1.2.840.10008.5.1.4.1.1.12.2.1')
PositronEmissionTomographyImageStorage = UID(
    '1.2.840.10008.5.1.4.1.1.128')
LegacyConvertedEnhancedPETImageStorage = UID(
    '1.2.840.10008.5.1.4.1.1.128.1')
XRay3DAngiographicImageStorage = UID(
    '1.2.840.10008.5.1.4.1.1.13.1.1')
XRay3DCraniofacialImageStorage = UID(
    '1.2.840.10008.5.1.4.1.1.13.1.2')
BreastTomosynthesisImageStorage = UID(
    '1.2.840.10008.5.1.4.1.1.13.1.3')
BreastProjectionXRayImageStorageForPresentation = UID(
    '1.2.840.10008.5.1.4.1.1.13.1.4')
BreastProjectionXRayImageStorageForProcessing = UID(
    '1.2.840.10008.5.1.4.1.1.13.1.5')
EnhancedPETImageStorage = UID(
    '1.2.840.10008.5.1.4.1.1.130')
BasicStructuredDisplayStorage = UID(
    '1.2.840.10008.5.1.4.1.1.131')
IntravascularOpticalCoherenceTomographyImageStorageForPresentation = UID(
    '1.2.840.10008.5.1.4.1.1.14.1')
IntravascularOpticalCoherenceTomographyImageStorageForProcessing = UID(
    '1.2.840.10008.5.1.4.1.1.14.2')
CTImageStorage = UID(
    '1.2.840.10008.5.1.4.1.1.2')
EnhancedCTImageStorage = UID(
    '1.2.840.10008.5.1.4.1.1.2.1')
LegacyConvertedEnhancedCTImageStorage = UID(
    '1.2.840.10008.5.1.4.1.1.2.2')
NuclearMedicineImageStorage = UID(
    '1.2.840.10008.5.1.4.1.1.20')
CTDefinedProcedureProtocolStorage = UID(
    '1.2.840.10008.5.1.4.1.1.200.1')
CTPerformedProcedureProtocolStorage = UID(
    '1.2.840.10008.5.1.4.1.1.200.2')
ProtocolApprovalStorage = UID(
    '1.2.840.10008.5.1.4.1.1.200.3')
UltrasoundMultiFrameImageStorage = UID(
    '1.2.840.10008.5.1.4.1.1.3.1')
ParametricMapStorage = UID(
    '1.2.840.10008.5.1.4.1.1.30')
MRImageStorage = UID(
    '1.2.840.10008.5.1.4.1.1.4')
EnhancedMRImageStorage = UID(
    '1.2.840.10008.5.1.4.1.1.4.1')
MRSpectroscopyStorage = UID(
    '1.2.840.10008.5.1.4.1.1.4.2')
EnhancedMRColorImageStorage = UID(
    '1.2.840.10008.5.1.4.1.1.4.3')
LegacyConvertedEnhancedMRImageStorage = UID(
    '1.2.840.10008.5.1.4.1.1.4.4')
RTImageStorage = UID(
    '1.2.840.10008.5.1.4.1.1.481.1')
RTPhysicianIntentStorage = UID(
    '1.2.840.10008.5.1.4.1.1.481.10')
RTSegmentAnnotationStorage = UID(
    '1.2.840.10008.5.1.4.1.1.481.11')
RTRadiationSetStorage = UID(
    '1.2.840.10008.5.1.4.1.1.481.12')
CArmPhotonElectronRadiationStorage = UID(
    '1.2.840.10008.5.1.4.1.1.481.13')
TomotherapeuticRadiationStorage = UID(
    '1.2.840.10008.5.1.4.1.1.481.14')
RoboticArmRadiationStorage = UID(
    '1.2.840.10008.5.1.4.1.1.481.15')
RTRadiationRecordSetStorage = UID(
    '1.2.840.10008.5.1.4.1.1.481.16')
RTRadiationSalvageRecordStorage = UID(
    '1.2.840.10008.5.1.4.1.1.481.17')
TomotherapeuticRadiationRecordStorage = UID(
    '1.2.840.10008.5.1.4.1.1.481.18')
CArmPhotonElectronRadiationRecordStorage = UID(
    '1.2.840.10008.5.1.4.1.1.481.19')
RTDoseStorage = UID(
    '1.2.840.10008.5.1.4.1.1.481.2')
RoboticRadiationRecordStorage = UID(
    '1.2.840.10008.5.1.4.1.1.481.20')
RTStructureSetStorage = UID(
    '1.2.840.10008.5.1.4.1.1.481.3')
RTBeamsTreatmentRecordStorage = UID(
    '1.2.840.10008.5.1.4.1.1.481.4')
RTPlanStorage = UID(
    '1.2.840.10008.5.1.4.1.1.481.5')
RTBrachyTreatmentRecordStorage = UID(
    '1.2.840.10008.5.1.4.1.1.481.6')
RTTreatmentSummaryRecordStorage = UID(
    '1.2.840.10008.5.1.4.1.1.481.7')
RTIonPlanStorage = UID(
    '1.2.840.10008.5.1.4.1.1.481.8')
RTIonBeamsTreatmentRecordStorage = UID(
    '1.2.840.10008.5.1.4.1.1.481.9')
DICOSCTImageStorage = UID(
    '1.2.840.10008.5.1.4.1.1.501.1')
DICOSDigitalXRayImageStorageForPresentation = UID(
    '1.2.840.10008.5.1.4.1.1.501.2.1')
DICOSDigitalXRayImageStorageForProcessing = UID(
    '1.2.840.10008.5.1.4.1.1.501.2.2')
DICOSThreatDetectionReportStorage = UID(
    '1.2.840.10008.5.1.4.1.1.501.3')
DICOS2DAITStorage = UID(
    '1.2.840.10008.5.1.4.1.1.501.4')
DICOS3DAITStorage = UID(
    '1.2.840.10008.5.1.4.1.1.501.5')
DICOSQuadrupoleResonanceStorage = UID(
    '1.2.840.10008.5.1.4.1.1.501.6')
UltrasoundImageStorage = UID(
    '1.2.840.10008.5.1.4.1.1.6.1')
EnhancedUSVolumeStorage = UID(
    '1.2.840.10008.5.1.4.1.1.6.2')
EddyCurrentImageStorage = UID(
    '1.2.840.10008.5.1.4.1.1.601.1')
EddyCurrentMultiFrameImageStorage = UID(
    '1.2.840.10008.5.1.4.1.1.601.2')
RawDataStorage = UID(
    '1.2.840.10008.5.1.4.1.1.66')
SpatialRegistrationStorage = UID(
    '1.2.840.10008.5.1.4.1.1.66.1')
SpatialFiducialsStorage = UID(
    '1.2.840.10008.5.1.4.1.1.66.2')
DeformableSpatialRegistrationStorage = UID(
    '1.2.840.10008.5.1.4.1.1.66.3')
SegmentationStorage = UID(
    '1.2.840.10008.5.1.4.1.1.66.4')
SurfaceSegmentationStorage = UID(
    '1.2.840.10008.5.1.4.1.1.66.5')
TractographyResultsStorage = UID(
    '1.2.840.10008.5.1.4.1.1.66.6')
RealWorldValueMappingStorage = UID(
    '1.2.840.10008.5.1.4.1.1.67')
SurfaceScanMeshStorage = UID(
    '1.2.840.10008.5.1.4.1.1.68.1')
SurfaceScanPointCloudStorage = UID(
    '1.2.840.10008.5.1.4.1.1.68.2')
SecondaryCaptureImageStorage = UID(
    '1.2.840.10008.5.1.4.1.1.7')
MultiFrameSingleBitSecondaryCaptureImageStorage = UID(
    '1.2.840.10008.5.1.4.1.1.7.1')
MultiFrameGrayscaleByteSecondaryCaptureImageStorage = UID(
    '1.2.840.10008.5.1.4.1.1.7.2')
MultiFrameGrayscaleWordSecondaryCaptureImageStorage = UID(
    '1.2.840.10008.5.1.4.1.1.7.3')
MultiFrameTrueColorSecondaryCaptureImageStorage = UID(
    '1.2.840.10008.5.1.4.1.1.7.4')
VLEndoscopicImageStorage = UID(
    '1.2.840.10008.5.1.4.1.1.77.1.1')
VideoEndoscopicImageStorage = UID(
    '1.2.840.10008.5.1.4.1.1.77.1.1.1')
VLMicroscopicImageStorage = UID(
    '1.2.840.10008.5.1.4.1.1.77.1.2')
VideoMicroscopicImageStorage = UID(
    '1.2.840.10008.5.1.4.1.1.77.1.2.1')
VLSlideCoordinatesMicroscopicImageStorage = UID(
    '1.2.840.10008.5.1.4.1.1.77.1.3')
VLPhotographicImageStorage = UID(
    '1.2.840.10008.5.1.4.1.1.77.1.4')
VideoPhotographicImageStorage = UID(
    '1.2.840.10008.5.1.4.1.1.77.1.4.1')
OphthalmicPhotography8BitImageStorage = UID(
    '1.2.840.10008.5.1.4.1.1.77.1.5.1')
OphthalmicPhotography16BitImageStorage = UID(
    '1.2.840.10008.5.1.4.1.1.77.1.5.2')
StereometricRelationshipStorage = UID(
    '1.2.840.10008.5.1.4.1.1.77.1.5.3')
OphthalmicTomographyImageStorage = UID(
    '1.2.840.10008.5.1.4.1.1.77.1.5.4')
WideFieldOphthalmicPhotographyStereographicProjectionImageStorage = UID(
    '1.2.840.10008.5.1.4.1.1.77.1.5.5')
WideFieldOphthalmicPhotography3DCoordinatesImageStorage = UID(
    '1.2.840.10008.5.1.4.1.1.77.1.5.6')
OphthalmicOpticalCoherenceTomographyEnFaceImageStorage = UID(
    '1.2.840.10008.5.1.4.1.1.77.1.5.7')
OphthalmicOpticalCoherenceTomographyBscanVolumeAnalysisStorage = UID(
    '1.2.840.10008.5.1.4.1.1.77.1.5.8')
VLWholeSlideMicroscopyImageStorage = UID(
    '1.2.840.10008.5.1.4.1.1.77.1.6')
LensometryMeasurementsStorage = UID(
    '1.2.840.10008.5.1.4.1.1.78.1')
AutorefractionMeasurementsStorage = UID(
    '1.2.840.10008.5.1.4.1.1.78.2')
KeratometryMeasurementsStorage = UID(
    '1.2.840.10008.5.1.4.1.1.78.3')
SubjectiveRefractionMeasurementsStorage = UID(
    '1.2.840.10008.5.1.4.1.1.78.4')
VisualAcuityMeasurementsStorage = UID(
    '1.2.840.10008.5.1.4.1.1.78.5')
SpectaclePrescriptionReportStorage = UID(
    '1.2.840.10008.5.1.4.1.1.78.6')
OphthalmicAxialMeasurementsStorage = UID(
    '1.2.840.10008.5.1.4.1.1.78.7')
IntraocularLensCalculationsStorage = UID(
    '1.2.840.10008.5.1.4.1.1.78.8')
MacularGridThicknessAndVolumeReportStorage = UID(
    '1.2.840.10008.5.1.4.1.1.79.1')
OphthalmicVisualFieldStaticPerimetryMeasurementsStorage = UID(
    '1.2.840.10008.5.1.4.1.1.80.1')
OphthalmicThicknessMapStorage = UID(
    '1.2.840.10008.5.1.4.1.1.81.1')
CornealTopographyMapStorage = UID(
    '1.2.840.10008.5.1.4.1.1.82.1')
BasicTextSRStorage = UID(
    '1.2.840.10008.5.1.4.1.1.88.11')
EnhancedSRStorage = UID(
    '1.2.840.10008.5.1.4.1.1.88.22')
ComprehensiveSRStorage = UID(
    '1.2.840.10008.5.1.4.1.1.88.33')
Comprehensive3DSRStorage = UID(
    '1.2.840.10008.5.1.4.1.1.88.34')
ExtensibleSRStorage = UID(
    '1.2.840.10008.5.1.4.1.1.88.35')
ProcedureLogStorage = UID(
    '1.2.840.10008.5.1.4.1.1.88.40')
MammographyCADSRStorage = UID(
    '1.2.840.10008.5.1.4.1.1.88.50')
KeyObjectSelectionDocumentStorage = UID(
    '1.2.840.10008.5.1.4.1.1.88.59')
ChestCADSRStorage = UID(
    '1.2.840.10008.5.1.4.1.1.88.65')
XRayRadiationDoseSRStorage = UID(
    '1.2.840.10008.5.1.4.1.1.88.67')
RadiopharmaceuticalRadiationDoseSRStorage = UID(
    '1.2.840.10008.5.1.4.1.1.88.68')
ColonCADSRStorage = UID(
    '1.2.840.10008.5.1.4.1.1.88.69')
ImplantationPlanSRStorage = UID(
    '1.2.840.10008.5.1.4.1.1.88.70')
AcquisitionContextSRStorage = UID(
    '1.2.840.10008.5.1.4.1.1.88.71')
SimplifiedAdultEchoSRStorage = UID(
    '1.2.840.10008.5.1.4.1.1.88.72')
PatientRadiationDoseSRStorage = UID(
    '1.2.840.10008.5.1.4.1.1.88.73')
PlannedImagingAgentAdministrationSRStorage = UID(
    '1.2.840.10008.5.1.4.1.1.88.74')
PerformedImagingAgentAdministrationSRStorage = UID(
    '1.2.840.10008.5.1.4.1.1.88.75')
TwelveLeadECGWaveformStorage = UID(
    '1.2.840.10008.5.1.4.1.1.9.1.1')
GeneralECGWaveformStorage = UID(
    '1.2.840.10008.5.1.4.1.1.9.1.2')
AmbulatoryECGWaveformStorage = UID(
    '1.2.840.10008.5.1.4.1.1.9.1.3')
HemodynamicWaveformStorage = UID(
    '1.2.840.10008.5.1.4.1.1.9.2.1')
CardiacElectrophysiologyWaveformStorage = UID(
    '1.2.840.10008.5.1.4.1.1.9.3.1')
BasicVoiceAudioWaveformStorage = UID(
    '1.2.840.10008.5.1.4.1.1.9.4.1')
GeneralAudioWaveformStorage = UID(
    '1.2.840.10008.5.1.4.1.1.9.4.2')
ArterialPulseWaveformStorage = UID(
    '1.2.840.10008.5.1.4.1.1.9.5.1')
RespiratoryWaveformStorage = UID(
    '1.2.840.10008.5.1.4.1.1.9.6.1')
MultichannelRespiratoryWaveformStorage = UID(
    '1.2.840.10008.5.1.4.1.1.9.6.2')
RoutineScalpElectroencephalogramWaveformStorage = UID(
    '1.2.840.10008.5.1.4.1.1.9.7.1')
ElectromyogramWaveformStorage = UID(
    '1.2.840.10008.5.1.4.1.1.9.7.2')
ElectrooculogramWaveformStorage = UID(
    '1.2.840.10008.5.1.4.1.1.9.7.3')
SleepElectroencephalogramWaveformStorage = UID(
    '1.2.840.10008.5.1.4.1.1.9.7.4')
BodyPositionWaveformStorage = UID(
    '1.2.840.10008.5.1.4.1.1.9.8.1')
ContentAssessmentResultsStorage = UID(
    '1.2.840.10008.5.1.4.1.1.90.1')
RTBrachyApplicationSetupDeliveryInstructionStorage = UID(
    '1.2.840.10008.5.1.4.34.10')
RTBeamsDeliveryInstructionStorage = UID(
    '1.2.840.10008.5.1.4.34.7')
HangingProtocolStorage = UID(
    '1.2.840.10008.5.1.4.38.1')
ColorPaletteStorage = UID(
    '1.2.840.10008.5.1.4.39.1')
GenericImplantTemplateStorage = UID(
    '1.2.840.10008.5.1.4.43.1')
ImplantAssemblyTemplateStorage = UID(
    '1.2.840.10008.5.1.4.44.1')
ImplantTemplateGroupStorage = UID(
    '1.2.840.10008.5.1.4.45.1')



# Copyright 2008-2020 pydicom authors. See LICENSE file for details.
"""Compatibility functions for previous Python 2 support"""

# Python 2 only - warn and mark this as deprecated.
import warnings

from pydicom import config

if config._use_future:
    raise ImportError(
        "Pydicom Future Error: compat module will be removed in pydicom 3.0"
    )

warnings.warn(
    "Starting in pydicom 3.0, the compat module (used for Python 2)"
    " will be removed",
    DeprecationWarning
)

# Text types
text_type = str
string_types = (str, )
char_types = (str, bytes)
number_types = (int, )
int_type = int



# Copyright 2008-2018 pydicom authors. See LICENSE file for details.

import os
import sys
import pytest
import pydicom
from pydicom.filereader import dcmread
from pydicom.data import get_testdata_file

pillow_missing_message = ("pillow is not available "
                          "in this test environment")
pillow_present_message = "pillow is being tested"
gdcm_missing_message = "GDCM is not available in this test environment"
numpy_missing_message = ("numpy is not available "
                         "in this test environment")
jpeg_ls_missing_message = ("jpeg_ls is not available "
                           "in this test environment")


try:
    import pydicom.pixel_data_handlers.numpy_handler as numpy_handler
    HAVE_NP = numpy_handler.HAVE_NP
except ImportError:
    HAVE_NP = False
    numpy_handler = None

try:
    import pydicom.pixel_data_handlers.pillow_handler as pillow_handler
    HAVE_PIL = pillow_handler.HAVE_PIL
    HAVE_JPEG = pillow_handler.HAVE_JPEG
    HAVE_JPEG2K = pillow_handler.HAVE_JPEG2K
except ImportError:
    HAVE_PIL = False
    pillow_handler = None
    HAVE_JPEG = False
    HAVE_JPEG2K = False

try:
    import pydicom.pixel_data_handlers.jpeg_ls_handler as jpeg_ls_handler
    HAVE_JPEGLS = jpeg_ls_handler.HAVE_JPEGLS
except ImportError:
    jpeg_ls_handler = None
    HAVE_JPEGLS = False

try:
    import pydicom.pixel_data_handlers.gdcm_handler as gdcm_handler
    HAVE_GDCM = gdcm_handler.HAVE_GDCM
except ImportError:
    gdcm_handler = None
    HAVE_GDCM = False

mr_name = get_testdata_file("MR_small.dcm")
jpeg_ls_lossless_name = get_testdata_file("MR_small_jpeg_ls_lossless.dcm")
emri_name = get_testdata_file("emri_small.dcm")
emri_jpeg_ls_lossless = get_testdata_file(
    "emri_small_jpeg_ls_lossless.dcm")
dir_name = os.path.dirname(sys.argv[0])
save_dir = os.getcwd()


class Test_JPEG_LS_Lossless_transfer_syntax():
    def setup_method(self, method):
        self.jpeg_ls_lossless = dcmread(jpeg_ls_lossless_name)
        self.mr_small = dcmread(mr_name)
        self.emri_jpeg_ls_lossless = dcmread(emri_jpeg_ls_lossless)
        self.emri_small = dcmread(emri_name)
        self.original_handlers = pydicom.config.pixel_data_handlers

    def teardown_method(self, method):
        pydicom.config.pixel_data_handlers = self.original_handlers

    @pytest.mark.skipif(not HAVE_NP, reason=numpy_missing_message)
    def test_read_mr_with_numpy(self):
        pydicom.config.pixel_data_handlers = [numpy_handler]
        msg = (
            r"Unable to decode pixel data with a transfer syntax UID of "
            r"'1.2.840.10008.1.2.4.80' \(JPEG-LS Lossless Image Compression\) "
            r"as there are no pixel data handlers available."
        )
        with pytest.raises(NotImplementedError, match=msg):
            self.jpeg_ls_lossless.pixel_array

    @pytest.mark.skipif(not HAVE_NP, reason=numpy_missing_message)
    def test_read_emri_with_numpy(self):
        pydicom.config.pixel_data_handlers = [numpy_handler]
        msg = (
            r"Unable to decode pixel data with a transfer syntax UID of "
            r"'1.2.840.10008.1.2.4.80' \(JPEG-LS Lossless Image Compression\) "
            r"as there are no pixel data handlers available."
        )
        with pytest.raises(NotImplementedError, match=msg):
            self.emri_jpeg_ls_lossless.pixel_array

    @pytest.mark.skipif(not HAVE_PIL, reason=pillow_missing_message)
    def test_read_mr_with_pillow(self):
        pydicom.config.pixel_data_handlers = [pillow_handler]
        msg = (
            r"Unable to decode pixel data with a transfer syntax UID of "
            r"'1.2.840.10008.1.2.4.80' \(JPEG-LS Lossless Image Compression\) "
            r"as there are no pixel data handlers available."
        )
        with pytest.raises(NotImplementedError, match=msg):
            self.jpeg_ls_lossless.pixel_array

    @pytest.mark.skipif(not HAVE_PIL, reason=pillow_missing_message)
    def test_read_emri_with_pillow(self):
        pydicom.config.pixel_data_handlers = [pillow_handler]
        msg = (
            r"Unable to decode pixel data with a transfer syntax UID of "
            r"'1.2.840.10008.1.2.4.80' \(JPEG-LS Lossless Image Compression\) "
            r"as there are no pixel data handlers available."
        )
        with pytest.raises(NotImplementedError, match=msg):
            self.emri_jpeg_ls_lossless.pixel_array

    @pytest.mark.skipif(not HAVE_GDCM, reason=gdcm_missing_message)
    def test_read_mr_with_gdcm(self):
        pydicom.config.pixel_data_handlers = [numpy_handler, gdcm_handler]
        a = self.jpeg_ls_lossless.pixel_array
        b = self.mr_small.pixel_array
        assert a.mean() == b.mean(), \
            "using GDCM Decoded pixel data is not " \
            "all {0} (mean == {1})".format(b.mean(), a.mean())

    @pytest.mark.skipif(not HAVE_GDCM, reason=gdcm_missing_message)
    def test_read_emri_with_gdcm(self):
        pydicom.config.pixel_data_handlers = [numpy_handler, gdcm_handler]
        a = self.emri_jpeg_ls_lossless.pixel_array
        b = self.emri_small.pixel_array
        assert a.mean() == b.mean(), \
            "using GDCM Decoded pixel data is not " \
            "all {0} (mean == {1})".format(b.mean(), a.mean())

    @pytest.mark.skipif(not HAVE_JPEGLS, reason=jpeg_ls_missing_message)
    def test_read_mr_with_jpeg_ls(self):
        pydicom.config.pixel_data_handlers = [numpy_handler, jpeg_ls_handler]
        a = self.jpeg_ls_lossless.pixel_array
        b = self.mr_small.pixel_array
        assert a.mean() == b.mean(), \
            "using jpeg_ls decoded pixel data is not " \
            "all {0} (mean == {1})".format(b.mean(), a.mean())

    @pytest.mark.skipif(not HAVE_JPEGLS, reason=jpeg_ls_missing_message)
    def test_read_emri_with_jpeg_ls(self):
        pydicom.config.pixel_data_handlers = [numpy_handler, jpeg_ls_handler]
        a = self.emri_jpeg_ls_lossless.pixel_array
        b = self.emri_small.pixel_array
        assert a.mean() == b.mean(), \
            "using jpeg_ls decoded pixel data is not all {0} " \
            "(mean == {1})".format(b.mean(), a.mean())

    def test_read_mr_without_any_handler(self):
        pydicom.config.pixel_data_handlers = []
        msg = (
            r"Unable to decode pixel data with a transfer syntax UID of "
            r"'1.2.840.10008.1.2.4.80' \(JPEG-LS Lossless Image Compression\) "
            r"as there are no pixel data handlers available."
        )
        with pytest.raises(NotImplementedError, match=msg):
            self.jpeg_ls_lossless.pixel_array

    def test_read_emri_without_any_handler(self):
        pydicom.config.pixel_data_handlers = []
        msg = (
            r"Unable to decode pixel data with a transfer syntax UID of "
            r"'1.2.840.10008.1.2.4.80' \(JPEG-LS Lossless Image Compression\) "
            r"as there are no pixel data handlers available."
        )
        with pytest.raises(NotImplementedError, match=msg):
            self.emri_jpeg_ls_lossless.pixel_array
